#main_window {
}

#gpg_key_list {
  padding: 0.1em;
}

GtkListBoxRow:nth-child(even) {
  background-color: #F7F0F0;
}

GtkListBoxRow:nth-child(odd) {
  background-color: #FFF;
}

GtkListBoxRow:hover,
GtkListBoxRow:active,
GtkListBoxRow:selected,
GtkListBoxRow:focus {
  background-color: #D0D0D7;
  color: #000;
}
